
# q-expansion of newform 700.2.r.d, downloaded from the LMFDB on 15 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 700
weight = 2
poly_data = [81, 0, -198, 0, 403, 0, -180, 0, 59, 0, -9, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[-60864, 0, 90554, 0, -27234, 0, 7472, 0, -767, 0, 117, 0], 21995], [[240966, 0, -490451, 0, 202311, 0, -71803, 0, 10953, 0, -1298, 0], 197955], [[-186237, 0, 142112, 0, -98847, 0, 28501, 0, -5466, 0, 461, 0], 65985], [[0, -729693, 0, 474413, 0, -169668, 0, 40474, 0, -6174, 0, 569], 197955], [[-68439, 0, -6336, 0, 12896, 0, -9933, 0, 1888, 0, -288, 0], 21995], [[0, 98293, 0, 1782, 0, -3627, 0, 3481, 0, -531, 0, 81], 21995], [[0, 90434, 0, 6336, 0, -12896, 0, 9933, 0, -1888, 0, 288], 65985], [[-271656, 0, 552916, 0, -273351, 0, 80948, 0, -12348, 0, 1138, 0], 65985], [[0, 445716, 0, 506489, 0, -234954, 0, 103132, 0, -15732, 0, 2027], 197955], [[0, -2916, 0, 26581, 0, -13716, 0, 4868, 0, -783, 0, 88], 7155], [[0, 43011, 0, -450860, 0, 202311, 0, -71803, 0, 10953, 0, -1298], 39591]]

hecke_ring_character_values = [[351, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [477, [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [101, [0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 1, 0, 0, 0, 0, 0, -1, 0], [-1, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 1], [0, 0, 0, 0, -1, 0, 0, -4, 0, -2, 4, -1], [0, -2, 0, -1, 0, 1, 0, 0, 1, 0, 0, 0], [0, 0, 0, 0, 1, 0, 2, 0, 0, -1, 0, 0], [0, -1, 0, -2, 0, -1, 0, 0, 1, 0, 0, 0], [1, 1, -1, -1, 0, -2, 0, 0, 2, 0, 0, 0], [0, 0, 0, 0, 0, 0, 3, 1, 0, 0, 0, 0], [2, 1, 0, 2, 0, -4, 0, 0, -1, 0, 0, 0], [0, 0, 0, 0, 2, 0, 1, 0, 0, 1, -4, 2], [0, 0, 0, 0, 1, 0, 0, 1, 0, -1, 0, 0], [0, 0, 0, 0, -1, 0, -3, 5, 0, -2, -5, -4], [-2, 0, 2, 0, 0, -1, 0, 0, 1, 0, 0, 0], [0, 2, -7, 1, 0, -1, 0, 0, -3, 0, 0, 0], [0, 0, 0, 0, 0, 0, -2, -6, 0, 0, 6, -2], [-7, 0, 0, 0, 0, -2, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 4, 0, 0, -4, 0], [0, 2, -2, 1, 0, -1, 0, 0, -4, 0, 0, 0], [0, 0, 0, 0, 2, 0, 1, 0, 0, 1, 10, 5], [0, 2, -5, 1, 0, -1, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, -4, 0, -2, 0, 0, -2, -4, -3], [-3, -3, 3, 3, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, -2, 0, 1, -2, 0, 2, 0, 0], [0, 0, 0, 0, 1, 0, 2, -12, 0, -1, 0, 0], [-3, 2, 3, -2, 0, 1, 0, 0, -1, 0, 0, 0], [0, 0, 0, 0, -6, 0, -3, 0, 0, -3, 1, -1], [0, 0, 0, 0, 2, 0, 1, 0, 0, 1, -2, -2], [0, -6, 1, -3, 0, 3, 0, 0, -1, 0, 0, 0], [0, 0, 0, 0, -1, 0, -5, 0, 0, -2, 0, -6], [11, 2, 0, 4, 0, 1, 0, 0, -2, 0, 0, 0], [0, 6, 6, 3, 0, -3, 0, 0, 3, 0, 0, 0], [-8, 1, 8, -1, 0, -2, 0, 0, 2, 0, 0, 0], [0, 0, 0, 0, 2, 0, -3, -2, 0, 4, 2, -1], [0, 0, 0, 0, -1, 0, -2, 11, 0, 1, 0, 0], [0, 0, 0, 0, -4, 0, -2, 0, 0, -2, 3, -3], [0, 0, 0, 0, -1, 0, -3, -13, 0, 1, 0, 0], [0, 2, 0, -2, 0, 2, 0, 0, -2, 0, 0, 0], [4, 1, 0, 2, 0, -1, 0, 0, -1, 0, 0, 0], [0, -4, -3, -2, 0, 2, 0, 0, -2, 0, 0, 0], [0, 0, 0, 0, -1, 0, -5, 10, 0, -2, -10, -6], [0, 0, 0, 0, -4, 0, -2, 0, 0, -2, 2, 1], [-1, 1, 1, -1, 0, -5, 0, 0, 5, 0, 0, 0], [7, -3, 0, -6, 0, 1, 0, 0, 3, 0, 0, 0], [0, 0, 0, 0, 4, 0, 2, 0, 0, 2, 8, -1], [0, 0, 0, 0, -3, 0, 2, -7, 0, -6, 7, -1], [0, 0, -8, 0, 0, 0, 0, 0, 1, 0, 0, 0], [0, 0, 0, 0, 2, 0, -2, 3, 0, -2, 0, 0], [-10, -1, 0, -2, 0, 3, 0, 0, 1, 0, 0, 0], [8, 3, -8, -3, 0, -1, 0, 0, 1, 0, 0, 0], [3, 3, 0, 6, 0, 3, 0, 0, -3, 0, 0, 0], [0, 0, 0, 0, 2, 0, 2, 16, 0, -2, 0, 0], [0, 0, 0, 0, -4, 0, -1, 4, 0, -8, -4, -5], [15, 4, -15, -4, 0, -2, 0, 0, 2, 0, 0, 0], [0, 2, -9, 1, 0, -1, 0, 0, 2, 0, 0, 0], [0, 0, 0, 0, 2, 0, 5, 0, 0, 4, 0, 7], [-10, 2, 0, 4, 0, 0, 0, 0, -2, 0, 0, 0], [0, 0, 0, 0, 2, 0, 7, 4, 0, 4, -4, 9], [0, 0, 0, 0, 2, 0, 1, 0, 0, 1, 17, -2], [0, 0, 0, 0, 10, 0, 5, 0, 0, 5, -8, 4], [0, -2, 0, 2, 0, 10, 0, 0, -10, 0, 0, 0], [0, 0, 0, 0, 0, 0, 3, -1, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -4, 5, 0, 0, 0, 0], [0, 4, 5, 2, 0, -2, 0, 0, -8, 0, 0, 0], [0, 0, 0, 0, 2, 0, 1, 0, 0, 1, 19, 7], [0, 0, 0, 0, -4, 0, -1, -2, 0, -8, 2, -5], [-7, -1, 0, -2, 0, 4, 0, 0, 1, 0, 0, 0], [0, 0, 0, 0, -5, 0, -4, 3, 0, -10, -3, -9], [0, -10, 5, -5, 0, 5, 0, 0, 5, 0, 0, 0], [0, 0, 0, 0, -1, 0, 0, 19, 0, -2, -19, -1], [0, 0, 0, 0, 3, 0, 3, -13, 0, -3, 0, 0], [3, -3, 0, -6, 0, 1, 0, 0, 3, 0, 0, 0], [0, 0, 0, 0, 4, 0, 11, -3, 0, -4, 0, 0], [-10, 3, 10, -3, 0, 7, 0, 0, -7, 0, 0, 0], [0, 0, 0, 0, 1, 0, 0, -7, 0, -1, 0, 0], [0, 0, 2, 0, 0, 0, 0, 0, -5, 0, 0, 0], [-3, -6, 3, 6, 0, 2, 0, 0, -2, 0, 0, 0], [6, -1, 0, -2, 0, -1, 0, 0, 1, 0, 0, 0], [2, 0, 0, 0, 0, 9, 0, 0, 0, 0, 0, 0], [-15, -4, 15, 4, 0, -1, 0, 0, 1, 0, 0, 0], [0, 0, 0, 0, 10, 0, 5, 0, 0, 5, -2, 3], [0, 2, -8, 1, 0, -1, 0, 0, 8, 0, 0, 0], [0, 0, 0, 0, 1, 0, -3, 7, 0, -1, 0, 0], [-15, 4, 0, 8, 0, -8, 0, 0, -4, 0, 0, 0], [0, 0, 0, 0, 5, 0, 8, 5, 0, -5, 0, 0], [-11, -1, 0, -2, 0, -5, 0, 0, 1, 0, 0, 0], [0, 0, 0, 0, -12, 0, -6, 0, 0, -6, 4, -12], [0, 0, 0, 0, -1, 0, -1, 25, 0, 1, 0, 0], [-15, -2, 15, 2, 0, -5, 0, 0, 5, 0, 0, 0], [0, 0, 0, 0, 3, 0, 0, 2, 0, 6, -2, 3], [-18, 4, 0, 8, 0, -2, 0, 0, -4, 0, 0, 0], [0, 8, -9, 4, 0, -4, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, -6, 0, -3, 0, 0, -3, -12, -3], [0, 12, 10, 6, 0, -6, 0, 0, 2, 0, 0, 0], [-2, -2, 2, 2, 0, -6, 0, 0, 6, 0, 0, 0], [0, 0, 0, 0, 0, 0, 2, -11, 0, 0, 0, 0], [0, 2, -3, 1, 0, -1, 0, 0, 2, 0, 0, 0], [0, 0, 0, 0, -8, 0, -4, 0, 0, -4, -6, -1], [0, 0, 0, 0, 3, 0, 1, 10, 0, 6, -10, 4], [0, 0, 0, 0, -2, 0, 0, 7, 0, -4, -7, -2], [0, 14, -24, 7, 0, -7, 0, 0, -5, 0, 0, 0], [30, 2, -30, -2, 0, -3, 0, 0, 3, 0, 0, 0], [0, 0, 0, 0, 3, 0, 14, -5, 0, 6, 5, 17], [0, 0, 0, 0, 2, 0, 1, 0, 0, 1, 5, 4], [0, 0, 0, 0, -2, 0, 2, -6, 0, 2, 0, 0], [-6, 0, 6, 0, 0, 6, 0, 0, -6, 0, 0, 0], [-6, -5, 0, -10, 0, 0, 0, 0, 5, 0, 0, 0], [0, 0, 0, 0, -1, 0, -1, -22, 0, 1, 0, 0], [0, 0, 0, 0, 5, 0, 0, 0, 0, 10, 0, 5], [0, 0, 0, 0, -6, 0, -3, 0, 0, -3, -19, 1], [-13, 4, 13, -4, 0, 4, 0, 0, -4, 0, 0, 0], [-7, 5, 0, 10, 0, -1, 0, 0, -5, 0, 0, 0], [-11, -1, 11, 1, 0, 10, 0, 0, -10, 0, 0, 0], [0, 0, 0, 0, -8, 0, -4, 0, 0, -4, -7, -10], [0, 0, 0, 0, -3, 0, -11, 7, 0, -6, -7, -14], [0, 0, 0, 0, -7, 0, 0, 11, 0, 7, 0, 0], [20, -4, 0, -8, 0, -3, 0, 0, 4, 0, 0, 0], [2, -2, -2, 2, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 4, 0, 2, 0, 0, 2, 2, 17], [0, 0, 0, 0, -3, 0, 3, 24, 0, 3, 0, 0], [0, 0, 0, 0, 1, 0, -3, -35, 0, 2, 35, -2], [0, -8, -7, -4, 0, 4, 0, 0, -5, 0, 0, 0], [10, -5, 0, -10, 0, 3, 0, 0, 5, 0, 0, 0], [0, 4, 16, 2, 0, -2, 0, 0, -3, 0, 0, 0], [0, -2, 39, -1, 0, 1, 0, 0, 5, 0, 0, 0], [0, 0, 0, 0, -4, 0, -2, 0, 0, -2, 21, -1], [0, 0, 0, 0, 1, 0, 7, 13, 0, -1, 0, 0], [19, 2, -19, -2, 0, 6, 0, 0, -6, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12, 15], [0, -12, 4, -6, 0, 6, 0, 0, 9, 0, 0, 0], [0, 0, 0, 0, 10, 0, 5, 0, 0, 5, 16, 13], [0, 4, 15, 2, 0, -2, 0, 0, 8, 0, 0, 0], [-31, 5, 0, 10, 0, -5, 0, 0, -5, 0, 0, 0], [0, 0, 0, 0, 7, 0, 5, 24, 0, 14, -24, 12], [0, 0, 0, 0, -5, 0, 1, 10, 0, -10, -10, -4], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -24, 3], [-6, -3, 6, 3, 0, 0, 0, 0, 0, 0, 0, 0], [-4, 3, 0, 6, 0, -12, 0, 0, -3, 0, 0, 0], [0, -6, 3, -3, 0, 3, 0, 0, 15, 0, 0, 0], [0, 0, 0, 0, 3, 0, -8, 11, 0, 6, -11, -5], [0, 0, 0, 0, -6, 0, -3, 0, 0, -3, -19, -11], [-13, 3, 13, -3, 0, -15, 0, 0, 15, 0, 0, 0], [-20, -2, 0, -4, 0, 6, 0, 0, 2, 0, 0, 0], [0, 0, 0, 0, -12, 0, -6, 0, 0, -6, -17, 3], [0, 0, 0, 0, -1, 0, 1, -42, 0, -2, 42, 0], [0, 4, -27, 2, 0, -2, 0, 0, -5, 0, 0, 0], [0, 0, 0, 0, 5, 0, 2, -23, 0, -5, 0, 0], [0, 0, 0, 0, 2, 0, -2, 4, 0, -2, 0, 0], [-28, -2, 0, -4, 0, -1, 0, 0, 2, 0, 0, 0], [0, 0, 0, 0, 20, 0, 10, 0, 0, 10, -3, 13], [0, 0, 0, 0, 3, 0, 10, 4, 0, -3, 0, 0], [0, 0, 0, 0, 0, 0, -6, -25, 0, 0, 25, -6], [-21, -3, 0, -6, 0, 6, 0, 0, 3, 0, 0, 0], [0, -10, -13, -5, 0, 5, 0, 0, 13, 0, 0, 0], [0, 0, 22, 0, 0, 0, 0, 0, 11, 0, 0, 0], [0, 0, 0, 0, 10, 0, 5, 0, 0, 5, 8, 9], [19, 4, -19, -4, 0, -3, 0, 0, 3, 0, 0, 0], [0, 0, 0, 0, 2, 0, 14, -14, 0, -2, 0, 0], [0, 0, 0, 0, -14, 0, -7, 0, 0, -7, 24, -12], [0, 0, 0, 0, -10, 0, -5, 0, 0, -5, 16, -5], [0, 8, 21, 4, 0, -4, 0, 0, -8, 0, 0, 0], [0, 0, 0, 0, 0, 0, -9, 27, 0, 0, -27, -9], [0, 0, 0, 0, 2, 0, 11, -26, 0, 4, 26, 13], [5, -6, -5, 6, 0, 2, 0, 0, -2, 0, 0, 0], [0, 0, 0, 0, -3, 0, 3, 5, 0, -6, -5, 0]]
